// Code generated by mockery v2.33.3. DO NOT EDIT.

package mocks

import (
	dbmodel "github.com/chroma/chroma-coordinator/internal/metastore/db/dbmodel"
	mock "github.com/stretchr/testify/mock"
)

// ISegmentMetadataDb is an autogenerated mock type for the ISegmentMetadataDb type
type ISegmentMetadataDb struct {
	mock.Mock
}

// DeleteAll provides a mock function with given fields:
func (_m *ISegmentMetadataDb) DeleteAll() error {
	ret := _m.Called()

	var r0 error
	if rf, ok := ret.Get(0).(func() error); ok {
		r0 = rf()
	} else {
		r0 = ret.Error(0)
	}

	return r0
}

// DeleteBySegmentID provides a mock function with given fields: segmentID
func (_m *ISegmentMetadataDb) DeleteBySegmentID(segmentID string) error {
	ret := _m.Called(segmentID)

	var r0 error
	if rf, ok := ret.Get(0).(func(string) error); ok {
		r0 = rf(segmentID)
	} else {
		r0 = ret.Error(0)
	}

	return r0
}

// DeleteBySegmentIDAndKeys provides a mock function with given fields: segmentID, keys
func (_m *ISegmentMetadataDb) DeleteBySegmentIDAndKeys(segmentID string, keys []string) error {
	ret := _m.Called(segmentID, keys)

	var r0 error
	if rf, ok := ret.Get(0).(func(string, []string) error); ok {
		r0 = rf(segmentID, keys)
	} else {
		r0 = ret.Error(0)
	}

	return r0
}

// Insert provides a mock function with given fields: in
func (_m *ISegmentMetadataDb) Insert(in []*dbmodel.SegmentMetadata) error {
	ret := _m.Called(in)

	var r0 error
	if rf, ok := ret.Get(0).(func([]*dbmodel.SegmentMetadata) error); ok {
		r0 = rf(in)
	} else {
		r0 = ret.Error(0)
	}

	return r0
}

// NewISegmentMetadataDb creates a new instance of ISegmentMetadataDb. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewISegmentMetadataDb(t interface {
	mock.TestingT
	Cleanup(func())
}) *ISegmentMetadataDb {
	mock := &ISegmentMetadataDb{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}
