// Code generated by mockery v2.33.3. DO NOT EDIT.

package mocks

import (
	dbmodel "github.com/chroma/chroma-coordinator/internal/metastore/db/dbmodel"
	mock "github.com/stretchr/testify/mock"
)

// IDatabaseDb is an autogenerated mock type for the IDatabaseDb type
type IDatabaseDb struct {
	mock.Mock
}

// DeleteAll provides a mock function with given fields:
func (_m *IDatabaseDb) DeleteAll() error {
	ret := _m.Called()

	var r0 error
	if rf, ok := ret.Get(0).(func() error); ok {
		r0 = rf()
	} else {
		r0 = ret.Error(0)
	}

	return r0
}

// GetAllDatabases provides a mock function with given fields:
func (_m *IDatabaseDb) GetAllDatabases() ([]*dbmodel.Database, error) {
	ret := _m.Called()

	var r0 []*dbmodel.Database
	var r1 error
	if rf, ok := ret.Get(0).(func() ([]*dbmodel.Database, error)); ok {
		return rf()
	}
	if rf, ok := ret.Get(0).(func() []*dbmodel.Database); ok {
		r0 = rf()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).([]*dbmodel.Database)
		}
	}

	if rf, ok := ret.Get(1).(func() error); ok {
		r1 = rf()
	} else {
		r1 = ret.Error(1)
	}

	return r0, r1
}

// GetDatabases provides a mock function with given fields: databaseName, tenantID
func (_m *IDatabaseDb) GetDatabases(databaseName string, tenantID string) ([]*dbmodel.Database, error) {
	ret := _m.Called(databaseName, tenantID)

	var r0 []*dbmodel.Database
	var r1 error
	if rf, ok := ret.Get(0).(func(string, string) ([]*dbmodel.Database, error)); ok {
		return rf(databaseName, tenantID)
	}
	if rf, ok := ret.Get(0).(func(string, string) []*dbmodel.Database); ok {
		r0 = rf(databaseName, tenantID)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).([]*dbmodel.Database)
		}
	}

	if rf, ok := ret.Get(1).(func(string, string) error); ok {
		r1 = rf(databaseName, tenantID)
	} else {
		r1 = ret.Error(1)
	}

	return r0, r1
}

// Insert provides a mock function with given fields: in
func (_m *IDatabaseDb) Insert(in *dbmodel.Database) error {
	ret := _m.Called(in)

	var r0 error
	if rf, ok := ret.Get(0).(func(*dbmodel.Database) error); ok {
		r0 = rf(in)
	} else {
		r0 = ret.Error(0)
	}

	return r0
}

// NewIDatabaseDb creates a new instance of IDatabaseDb. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewIDatabaseDb(t interface {
	mock.TestingT
	Cleanup(func())
}) *IDatabaseDb {
	mock := &IDatabaseDb{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}
